package Module::CoreList::More;

our $DATE = '2015-04-14'; # DATE
our $VERSION = '0.03'; # VERSION

use 5.010001;
use strict;
use warnings;

use Module::CoreList;

sub is_still_core {
    my $module = shift;
    $module = shift if eval { $module->isa(__PACKAGE__) } && @_ > 0 && defined($_[0]) && $_[0] =~ /^\w/;
    my ($module_version, $perl_version);

    $module_version = shift if @_ > 0;
    $perl_version   = @_ > 0 ? shift : $];

    my $first_rel; # first perl version where module is in core

  RELEASE:
    for my $rel (sort keys %Module::CoreList::delta) {
        my $delta = $Module::CoreList::delta{$rel};
        if ($first_rel) {
            # we have found the first release where module is included, check if
            # module is removed
            return 0 if $delta->{removed}{$module};
        } else {
            # we haven't found the first release where module is included
            if (exists $delta->{changed}{$module}) {
                my $modver = $delta->{changed}{$module};
                if (defined $module_version) {
                    if (version->parse($modver) >= version->parse($module_version)) {
                        $first_rel = $rel;
                    }
                } else {
                    $first_rel = $rel;
                }
                if ($first_rel) {
                    return 0 if $first_rel > $perl_version;
                }
            }
        }
    }

    # module has been included and never removed
    return 1 if $first_rel;

    # we never found the first release where module is first included
    0;
}

sub list_still_core_modules {
    my $class = shift if @_ && eval { $_[0]->isa(__PACKAGE__) };
    my $perl_version = @_ ? shift : $];

    my %added;
    my %removed;

  RELEASE:
    for my $rel (sort keys %Module::CoreList::delta) {
        my $delta = $Module::CoreList::delta{$rel};

        next unless $delta->{changed};
        for my $mod (keys %{$delta->{changed}}) {
            # module has been removed between perl_version..latest, skip
            next if $removed{$mod};

            if (exists $added{$mod}) {
                # module has been added in a previous version, update first
                # version
                $added{$mod} = $delta->{changed}{$mod} if $rel <= $perl_version;
            } else {
                # module is first added after perl_version, skip
                next if $rel > $perl_version;

                $added{$mod} = $delta->{changed}{$mod};
            }
        }
        next unless $delta->{removed};
        for my $mod (keys %{$delta->{removed}}) {
            delete $added{$mod};
            # module has been removed between perl_version..latest, mark it
            $removed{$mod}++ if $rel >= $perl_version;
        }

    }
    %added;
}

1;

# ABSTRACT: More functions for Module::CoreList

__END__

=pod

=encoding UTF-8

=head1 NAME

Module::CoreList::More - More functions for Module::CoreList

=head1 VERSION

This document describes version 0.03 of Module::CoreList::More (from Perl distribution Module-CoreList-More), released on 2015-04-14.

=head1 SYNOPSIS

 use Module::CoreList::More;

 # true, this module has always been in core since specified perl release
 Module::CoreList::More->is_still_core("Benchmark", 5.010001);

 # false, since CGI is removed in perl 5.021000
 Module::CoreList::More->is_still_core("CGI");

 # false, never been in core
 Module::CoreList::More->is_still_core("Foo");

 my %modules = list_still_core_modules(5.010001);

=head1 DESCRIPTION

This module is my experiment for providing more functionality to (or related to)
L<Module::CoreList>. Some ideas include: faster functions (for some use-cases),
more querying functions, more convenience functions. When I've got something
stable and useful to show for, I'll most probably suggest the appropriate
additions to Module::CoreList.

Below are random notes:

C<is_core()> is slow (+- 700/s on my office PC), I used to have a problem with
this, but forgot where and I got a workaround anyway. It's generally unoptimized
because C<is_still_core()> from this module is about 20x faster (see
L</"BENCHMARK">). We can also speed things up further e.g. by producing a cached
data structure of list of core modules for a certain Perl release (the data
structure in Module::CoreList are just list of Perl releases + date %released
and %delta which only lists differences of modules between Perl releases).

=head1 FUNCTIONS

These functions are not exported. They can be called as function (e.g.
C<Module::CoreList::More::is_still_core($name)> or as class method (e.g. C<<
Module::CoreList::More->is_still_core($name) >>.

=head2 is_still_core( MODULE, [ MODULE_VERSION, [ PERL_VERSION ] ] )

Like C<is_core>, but will also check that from PERL_VERSION up to the latest
known version, MODULE has never been removed from core.

Note/idea: could also be implemented by adding a fourth argument
MAX_PERL_VERSION to C<is_core>, defaulting to the latest known version.

=head2 list_still_core_modules([ PERL_VERSION ]) => %modules

List modules that are (still) in core from specified perl release to the latest.
Keys are module names, while values are versions of said modules in specified
perl release.

=head1 BENCHMARK

                                      Rate  is_core(Foo) is_core(Benchmark) is_core(CGI) is_still_core(Foo) is_still_core(Benchmark) is_still_core(CGI)
 is_core(Foo)             134.026+-0.052/s            --             -70.8%       -79.9%             -98.8%                   -98.9%             -99.0%
 is_core(Benchmark)         459.21+-0.61/s 242.63+-0.47%                 --       -31.2%             -96.0%                   -96.4%             -96.6%
 is_core(CGI)                667.83+-0.4/s        398.3%       45.43+-0.21%           --             -94.1%                   -94.8%             -95.0%
 is_still_core(Foo)            11341+-12/s  8361.9+-9.6%       2369.7+-4.2% 1598.2+-2.1%                 --                   -11.0%             -15.3%
 is_still_core(Benchmark)      12749+-16/s     9412+-13%       2676.2+-5.1% 1808.9+-2.7%       12.41+-0.19%                       --              -4.8%
 is_still_core(CGI)            13388+-15/s     9889+-12%       2815.4+-5.1% 1904.6+-2.6%       18.04+-0.18%              5.01+-0.18%                 --
 
                                             Rate list_still_core_modules(5.020002) list_still_core_modules(5.010001)
 list_still_core_modules(5.020002) 168.93+-0.68/s                                --                            -24.9%
 list_still_core_modules(5.010001)  224.91+-0.1/s                      33.14+-0.54%                                --

=head1 SEE ALSO

L<Module::CoreList>

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Module-CoreList-More>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Module-CoreList-More>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Module-CoreList-More>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
